#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Global static variables."""
from os import getenv
from os.path import expanduser, join

CONFIG_FILE = getenv(
    'POLICY_CONFIG',
    join(expanduser('~'), '.pacifica-policy', 'config.ini')
)
CHERRYPY_CONFIG = getenv(
    'POLICY_CPCONFIG',
    join(expanduser('~'), '.pacifica-policy', 'cpconfig.ini')
)
METADATA_CONNECT_ATTEMPTS = 40
METADATA_WAIT = 3

MATCH_VALIDATORS = {
    'project': r'[0-9]+[a-zA-Z]*',
    'user': r'[0-9]+',
    'transaction': r'[0-9]+'
}



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from cherrypy import tools, HTTPError
import requests
from pacifica.policy.config import get_config
from pacifica.policy.status.base import QueryBase


# pylint: disable=too-few-public-methods
class UserSearch(QueryBase):
    """Retrieves a set of projects for a given keyword set."""

    exposed = True

    @staticmethod
    def _get_users_for_keywords(kwargs, search_terms=None, option=None):
        """Return a list with all the projects involving this user."""
        md_url = '{0}/userinfo/search/{1}'.format(
            get_config().get('metadata', 'endpoint_url'), search_terms)
        if option is not None:
            md_url += '/{0}'.format(option)
        response = requests.get(url=md_url, params=kwargs)
        retval = response.json()
        if response.status_code == 200:
            return retval
        else:
            raise HTTPError(response.status_code)

    # CherryPy requires these named methods
    # Add HEAD (basically Get without returning body
    # pylint: disable=invalid-name
    @tools.json_out()
    def GET(self, search_terms=None, option=None, **kwargs):
        """CherryPy GET method."""
        return self._get_users_for_keywords(kwargs, search_terms, option)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the uploader policy."""
from os.path import join
from json import dumps, loads
from cherrypy.test import helper
from common_test import CommonCPSetup


class TestIngestPolicy(helper.CPWebCase, CommonCPSetup):
    """Test the uploader policy service."""

    PORT = 8181
    HOST = '127.0.0.1'
    headers = [('Content-Type', 'application/json')]

    def test_queries(self):
        """Test posting the queries."""
        valid_query = loads(
            open(join('test_files', 'ingest_base_query.json')).read())
        ret_data = self.get_json_page('/ingest', valid_query)
        self.assertFalse(ret_data is None)
        self.assertTrue('status' in ret_data)
        self.assertEqual(ret_data['status'], 'success')

        # change project to valid but he's an admin so this works
        valid_query[4]['key'] = 'Tag'
        valid_query[2]['value'] = '1234a'
        self.get_json_page('/ingest', valid_query)

        # change the project to be invalid => fails
        invalid_query = loads(
            open(join('test_files', 'ingest_base_query.json')).read())
        invalid_query[2]['value'] = '12'
        self.getPage('/ingest',
                     self.headers +
                     [('Content-Length', str(len(dumps(invalid_query))))],
                     'POST',
                     dumps(invalid_query))
        self.assertStatus('412 Precondition Failed')
        ret_data = loads(self.body.decode('UTF-8'))
        self.assertFalse(ret_data is None)
        self.assertTrue('message' in ret_data)

        # change instrument to be invalid => fails
        invalid_query = loads(
            open(join('test_files', 'ingest_base_query.json')).read())
        invalid_query[3]['value'] = 4321
        self.getPage('/ingest',
                     self.headers +
                     [('Content-Length', str(len(dumps(invalid_query))))],
                     'POST',
                     dumps(invalid_query))
        self.assertStatus('412 Precondition Failed')
        ret_data = loads(self.body.decode('UTF-8'))
        self.assertFalse(ret_data is None)
        self.assertTrue('message' in ret_data)

        # change the query so that the instrument xrefs and project xrefs fail (but for valid base entities)
        invalid_query = loads(
            open(join('test_files', 'ingest_base_query.json')).read())
        invalid_query[3]['value'] = 74  # instrument
        invalid_query[2]['value'] = u'1234c\u00e9'  # project
        invalid_query[1]['value'] = 12  # submitter
        self.getPage('/ingest',
                     self.headers +
                     [('Content-Length', str(len(dumps(invalid_query))))],
                     'POST',
                     dumps(invalid_query))
        self.assertStatus('412 Precondition Failed')
        ret_data = loads(self.body.decode('UTF-8'))
        self.assertFalse(ret_data is None)
        self.assertTrue('message' in ret_data)

        del valid_query[1]
        self.getPage('/ingest',
                     self.headers +
                     [('Content-Length', str(len(dumps(valid_query))))],
                     'POST',
                     dumps(valid_query))
        self.assertStatus('412 Precondition Failed')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the connections to external service."""
from unittest import TestCase
from json import dumps
import httpretty
from pacifica.policy.config import get_config
from pacifica.policy.root import Root


class TestMetaConnect(TestCase):
    """Test the metadata connection."""

    @httpretty.activate
    def test_meta_connect(self):
        """Test the try meta connect method."""
        httpretty.register_uri(
            httpretty.GET,
            get_config().get('metadata', 'status_url'),
            body=dumps([]),
            content_type='application/json'
        )
        Root.try_meta_connect(0)
        self.assertEqual(httpretty.last_request().method, 'GET')

    @httpretty.activate
    def test_meta_connect_failure(self):
        """Test the try meta connect method but fail."""
        httpretty.register_uri(
            httpretty.GET,
            get_config().get('metadata', 'status_url'),
            body='',
            status=500,
            content_type='application/json'
        )
        hit_exception = False
        try:
            Root.try_meta_connect(39)
        # pylint: disable=broad-except
        except Exception:
            hit_exception = True
        # pylint: enable=broad-except
        self.assertTrue(hit_exception)
        self.assertEqual(httpretty.last_request().method, 'GET')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from pacifica.policy.status.project.by_user import ProjectUserSearch
from pacifica.policy.status.project.search import ProjectKeywordSearch
from pacifica.policy.status.project.lookup import ProjectLookup


# pylint: disable=too-few-public-methods
class ProjectQuery(object):
    """CherryPy root object class."""

    exposed = False

    def __init__(self):
        """Create local objects for sub tree items."""
        self.by_user_id = ProjectUserSearch()
        self.search = ProjectKeywordSearch()
        self.by_project_id = ProjectLookup()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""The CherryPy rest object for the structure."""
from pacifica.policy.reporting.transaction.transaction_summary import TransactionSummary
from pacifica.policy.reporting.transaction.transaction_details import TransactionDetails


# pylint: disable=too-few-public-methods
class ReportingPolicy(object):
    """
    CherryPy root object class.

    not exposed by default the base objects are exposed.

    """

    exposed = False

    def __init__(self):
        """Create local objects to allow for import to work."""
        self.transaction_summary = TransactionSummary()
        self.transaction_details = TransactionDetails()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from cherrypy import tools
import requests
from pacifica.policy.config import get_config


# pylint: disable=too-few-public-methods
class TransactionSearch(object):
    """Retrieves a set of transactions for a given keyword set."""

    exposed = True

    @staticmethod
    def _get_transactions_for_keywords(kwargs, option=None):
        """Return a list with all the projects involving this user."""
        md_url = '{0}/transactioninfo/search/{1}'.format(
            get_config().get('metadata', 'endpoint_url'), option)
        response = requests.get(url=md_url, params=kwargs)

        return response.json()

    # CherryPy requires these named methods
    # Add HEAD (basically Get without returning body
    # pylint: disable=invalid-name
    @staticmethod
    @tools.json_out()
    def GET(option=None, **kwargs):
        """CherryPy GET method."""
        return TransactionSearch._get_transactions_for_keywords(kwargs, option)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Configuration reading and validation module."""
from os import getenv
try:
    from ConfigParser import SafeConfigParser
except ImportError:  # pragma: no cover python 2 vs 3 issue
    from configparser import ConfigParser as SafeConfigParser
from pacifica.policy.globals import CONFIG_FILE


def get_config():
    """
    Return the ConfigParser object with defaults set.

    Currently metadata API doesn't work with SQLite the queries are
    too complex and it only is supported with MySQL and PostgreSQL.
    """
    configparser = SafeConfigParser()
    configparser.add_section('policy')
    configparser.set(
        'policy', 'internal_url_format',
        getenv('INTERNAL_URL_FORMAT',
               'https://internal.example.com/{transaction}')
    )
    configparser.set(
        'policy', 'release_url_format',
        getenv('RELEASE_URL_FORMAT',
               'https://release.example.com/{transaction}')
    )
    configparser.set(
        'policy', 'doi_url_format',
        getenv('DOI_URL_FORMAT', 'https://dx.doi.org/{doi}')
    )
    configparser.set(
        'policy', 'cache_size',
        getenv(
            'CACHE_SIZE',
            '10000'
        )
    )
    configparser.set(
        'policy', 'admin_group',
        getenv(
            'ADMIN_GROUP',
            'admin'
        )
    )
    configparser.set(
        'policy', 'admin_group_id',
        getenv('ADMIN_GROUP_ID', '0')
    )
    configparser.set(
        'policy', 'admin_user_id',
        getenv('ADMIN_USER_ID', '0')
    )
    configparser.add_section('metadata')
    configparser.set(
        'metadata', 'endpoint_url',
        getenv(
            'METADATA_URL',
            'http://localhost:8121'
        )
    )
    configparser.set(
        'metadata', 'status_url',
        getenv(
            'STATUS_URL',
            'http://localhost:8121/groups'
        )
    )
    configparser.add_section('elasticsearch')
    configparser.set('elasticsearch', 'url', getenv(
        'ELASTIC_ENDPOINT', 'http://127.0.0.1:9200'))
    configparser.set('elasticsearch', 'index', getenv(
        'ELASTIC_INDEX', 'pacifica_search'))
    configparser.read(CONFIG_FILE)
    return configparser



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the ingest with httpretty."""
from os.path import join
from json import loads
import httpretty
from pacifica.policy.ingest.rest import IngestPolicy
from uploader_policy_test import TestUploader


class TestIngest(TestUploader):
    """Test the ingest policy with httpretty."""

    @httpretty.activate
    def test_failed_admin_id(self):
        """override this to test valid query."""
        super(TestIngest, self).test_failed_admin_id()
        valid_query = loads(
            open(join('test_files', 'ingest_base_query.json')).read())
        ipolicy = IngestPolicy()
        # pylint: disable=no-member
        # pylint: disable=protected-access
        ret = ipolicy._valid_query(valid_query)
        self.assertTrue(ret)
        valid_query[1]['value'] = 100
        valid_query[2]['value'] = u'1234b\u00e9'
        ret = ipolicy._valid_query(valid_query)
        self.assertTrue(ret)
        # pylint: enable=protected-access
        # pylint: enable=no-member



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Events rest module for the cherrypy endpoint."""
import cherrypy
from pacifica.policy.ingest.rest import IngestPolicy


# pylint: disable=too-few-public-methods
class EventsPolicy(IngestPolicy):
    """
    CherryPy Events Policy.

    This exposes whether a user can see an event from.
    """

    exposed = True

    @cherrypy.tools.json_out()
    @cherrypy.tools.json_in()
    # pylint: disable=arguments-differ
    def POST(self, username):
        """Pull the json content and validate the user can see the event."""
        event_obj = cherrypy.request.json
        userinfo = self._user_info_from_kwds(
            network_id=username, recursion_depth=0, recursion_limit=1)
        for rec in event_obj['data']:
            if rec['destinationTable'] == 'Transactions.submitter':
                rec['value'] = userinfo[0]['_id']
        return self._valid_query(event_obj['data'])
    # pylint: enable=arguments-differ
# pylint: disable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from cherrypy import tools
import requests
from pacifica.policy.validation import validate_user
from pacifica.policy.config import get_config


# pylint: disable=too-few-public-methods
class UserLookup(object):
    """Retrieves info for the specified user."""

    exposed = True

    @staticmethod
    def _get_user_info(user_id):
        """Return detailed info about a given user."""
        lookup_url = '{0}/userinfo/by_id/{1}'.format(
            get_config().get('metadata', 'endpoint_url'), user_id
        )
        return requests.get(url=lookup_url).json()

    # CherryPy requires these named methods
    # Add HEAD (basically Get without returning body
    # pylint: disable=invalid-name
    @staticmethod
    @tools.json_out()
    @validate_user()
    def GET(user_id=None):
        """CherryPy GET method."""
        return UserLookup._get_user_info(user_id)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Validation methods for various objects."""
from __future__ import absolute_import
import re
from functools import wraps
import cherrypy
from .globals import MATCH_VALIDATORS


def validate_user(index=0):
    """Validate the user id."""
    return validate_universal(index, 'user')


def validate_transaction(index=0):
    """Validate the transaction id."""
    return validate_universal(index, 'transaction')


def validate_project(index=0):
    """Validate the project id."""
    return validate_universal(index, 'project')


def _get_check_id(index, *args, **kwargs):
    """Return the check ID in args or kwargs."""
    try:
        return args[int(index)]
    except ValueError:
        return kwargs[str(index)]


def validate_universal(index, regex):
    """Decorator generator to validate project field."""
    def decorator(func):
        """Wrapped decorator."""
        @wraps(func)
        def wrapper(*args, **kwargs):
            """Wrapped function."""
            check_id = _get_check_id(index, *args, **kwargs)
            if not re.match(MATCH_VALIDATORS[regex], check_id):
                message = 'Provide an appropriate {0}_id value'.format(regex)
                raise cherrypy.HTTPError(
                    '400 Invalid Request', message)
            return func(*args, **kwargs)
        return wrapper
    return decorator



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy root object class."""
from __future__ import print_function
import sys
from time import sleep
from json import dumps
import requests
import cherrypy
from pacifica.policy.uploader.rest import UploaderPolicy
from pacifica.policy.status.rest import StatusPolicy
from pacifica.policy.ingest.rest import IngestPolicy
from pacifica.policy.events.rest import EventsPolicy
from pacifica.policy.reporting.rest import ReportingPolicy
from pacifica.policy.config import get_config
from pacifica.policy.globals import METADATA_CONNECT_ATTEMPTS, METADATA_WAIT


def error_page_default(**kwargs):
    """The default error page should always enforce json."""
    cherrypy.response.headers['Content-Type'] = 'application/json'
    json_str = dumps({
        'postdata': cherrypy.request.body.read().decode('UTF-8'),
        'status': kwargs['status'],
        'message': kwargs['message'],
        'traceback': kwargs['traceback'],
        'version': kwargs['version']
    }, sort_keys=True, indent=4)
    print(json_str, file=sys.stderr)
    return json_str


# pylint: disable=too-few-public-methods
class Root(object):
    """
    CherryPy root object class.

    not exposed by default the base objects are exposed
    """

    exposed = False

    def __init__(self):
        """Create the local objects we need."""
        self.uploader = UploaderPolicy()
        self.status = StatusPolicy()
        self.reporting = ReportingPolicy()
        self.ingest = IngestPolicy()
        self.events = EventsPolicy()

    @classmethod
    def try_meta_connect(cls, attempts=0):
        """Try to connect to the metadata service see if its there."""
        try:
            ret = requests.get(get_config().get('metadata', 'status_url'))
            if ret.status_code != 200:
                raise Exception(
                    'try_meta_connect: {0}\n'.format(ret.status_code)
                )
        # pylint: disable=broad-except
        except Exception:
            # pylint: enable=broad-except
            if attempts < METADATA_CONNECT_ATTEMPTS:
                sleep(METADATA_WAIT)
                attempts += 1
                cls.try_meta_connect(attempts)
            else:
                raise Exception
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the uploader policy."""
from __future__ import print_function
from json import dumps, loads
from cherrypy.test import helper
from pacifica.policy.uploader.rest import UploaderPolicy
from common_test import CommonCPSetup


class TestUploaderPolicy(helper.CPWebCase, CommonCPSetup):
    """Test the uploader policy service."""

    PORT = 8181
    HOST = '127.0.0.1'
    headers = [('Content-Type', 'application/json')]

    queries = {
        'user_query': {
            'query': {
                'user': -1,
                'from': 'users',
                'columns': ['last_name', 'first_name'],
                'where': {'network_id': 'dmlb2001'}
            },
            'answer': [
                {
                    'last_name': u'Brown\u00e9 Jr',
                    'first_name': u'David\u00e9'
                }
            ]
        },
        'user_query_no_where': {
            'query': {
                'user': 100,
                'from': 'users',
                'columns': ['last_name', 'first_name'],
                'where': {}
            },
            'answer': [
                {
                    'last_name': u'Brown\u00e9 Jr',
                    'first_name': u'David\u00e9'
                }
            ]
        },
        'user_query_with_project': {
            'query': {
                'user': 100,
                'from': 'users',
                'columns': ['last_name', 'first_name'],
                'where': {'project_id': '1234a', 'user': 10}
            },
            'answer': [
                {
                    'last_name': u'Brown\xe9 Jr',
                    'first_name': u'David\xe9'
                }
            ]
        },
        'project_query': {
            'query': {
                'user': 10,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {'project_id': '1234a'}
            },
            'answer': [
                {
                    '_id': '1234a',
                    'title': u'Pacifica D\xe9velopment (active no close)'
                }
            ]
        },
        'project_query_not_admin': {
            'query': {
                'user': 100,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {'project_id': u'1234b\u00e9'}
            },
            'answer': [
                {
                    '_id': u'1234b\u00e9',
                    'title': u'Pacifica D\xe9velopment'
                }
            ]
        },
        'project_query_user': {
            'query': {
                'user': 100,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {'_id': '1234a'}
            },
            'answer': [
                {
                    '_id': '1234a',
                    'title': u'Pacifica D\xe9velopment (active no close)'
                }
            ]
        },
        'project_query_admin': {
            'query': {
                'user': 10,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {'_id': '1234a'}
            },
            'answer': [
                {
                    '_id': '1234a',
                    'title': u'Pacifica D\xe9velopment (active no close)'
                }
            ]
        },
        'project_query_with_instruments': {
            'query': {
                'user': 11,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {'instrument_id': 74}
            },
            'answer': [
                {
                    '_id': u'1234c\u00e9',
                    'title': u'Pacifica D\xe9velopment (Alt)',
                }
            ]
        },
        'project_query_for_user': {
            'query': {
                'user': 10,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {}
            },
            'answer': [
                {
                    '_id': '1234a',
                    'title': u'Pacifica D\xe9velopment (active no close)'
                }
            ]
        },
        'project_query_for_non_admin_no_group': {
            'query': {
                'user': 100,
                'from': 'projects',
                'columns': ['_id', 'title'],
                'where': {'instrument_id': 75}
            },
            'answer': [
                {
                    '_id': u'1234b\u00e9',
                    'title': u'Pacifica D\xe9velopment'
                }
            ]
        },
        'instrument_query': {
            'query': {
                'user': 100,
                'from': 'instruments',
                'columns': ['_id', 'name'],
                'where': {}
            },
            'answer': [
                {
                    '_id': 54,
                    'name': u'NMR PROBES: Nittany Liquid Prob\xe9s'
                }
            ]
        },
        'instrument_query_with_id': {
            'query': {
                'user': 100,
                'from': 'instruments',
                'columns': ['_id', 'name'],
                'where': {'_id': 54}
            },
            'answer': [
                {
                    '_id': 54,
                    'name': u'NMR PROBES: Nittany Liquid Prob\xe9s'
                }
            ]
        },
        'instrument_query_from_project': {
            'query': {
                'user': 10,
                'from': 'instruments',
                'columns': ['_id', 'name'],
                'where': {'project_id': '1234a'}
            },
            'answer': [
                {
                    '_id': 54,
                    'name': u'NMR PROBES: Nittany Liquid Prob\xe9s'
                }
            ]
        },
        'instrument_query_from_project_bad': {
            'query': {
                'user': 100,
                'from': 'instruments',
                'columns': ['_id', 'name'],
                'where': {'project_id': '1234a'}
            },
            'answer': [
                {
                    '_id': 54,
                    'name': u'NMR PROBES: Nittany Liquid Prob\xe9s'
                }
            ]
        }
    }

    def test_queries(self):
        """Test posting the queries."""
        for title, value in self.queries.items():
            print(title)
            self.getPage('/uploader',
                         self.headers +
                         [('Content-Length', str(len(dumps(value['query']))))],
                         'POST',
                         dumps(value['query']))
            self.assertStatus('200 OK')
            self.assertHeader('Content-Type', 'application/json')
            if loads(self.body.decode('UTF-8')):
                answer = loads(self.body.decode('UTF-8'))
                if '_id' in answer[0]:
                    answer = sorted(answer, key=lambda i: i['_id'])
                answer = answer[0]
                for akey, avalue in value['answer'][0].items():
                    self.assertTrue(akey in answer)
                    self.assertEqual(avalue, answer[akey])
            else:
                self.assertEqual(len(loads(self.body.decode('UTF-8'))), 0)

    def test_bad_query(self):
        """Try to throw a bad query at the query select method."""
        upolicy = UploaderPolicy()
        hit_exception = False
        try:
            # pylint: disable=protected-access
            upolicy._query_select({})
            # pylint: enable=protected-access
        except KeyError:
            hit_exception = True
        self.assertTrue(hit_exception)
        hit_exception = False
        try:
            bad_query = {
                'user': 100,
                'from': 'foo',
                'where': {}
            }
            # pylint: disable=protected-access
            upolicy._query_select(bad_query)
            # pylint: enable=protected-access
        except TypeError:
            hit_exception = True
        self.assertTrue(hit_exception)

        bad_queries = [
            {
                'user': 'abcd'
            },
            {
                'user': 'abcd',
                'from': 'users',
                'columns': ['first_name'],
                'where': {'no_such_column': 'foo'}
            },
            {
                'user': 10,
                'from': 'foobar',
                'columns': ['first_name'],
                'where': {'network_id': 'foo'}
            }
        ]
        for bad_query in bad_queries:
            self.getPage('/uploader',
                         self.headers +
                         [('Content-Length', str(len(dumps(bad_query))))],
                         'POST',
                         dumps(bad_query))
            self.assertStatus('500 Internal Server Error')
            hit_exception = False
            try:
                loads(self.body.decode('UTF-8'))
            except ValueError:  # pragma no cover
                hit_exception = True
            self.assertFalse(hit_exception)

        # pylint: disable=protected-access
        self.assertFalse(upolicy._valid_query({'foo': 'bar'}))
        self.assertFalse(upolicy._valid_query({'user': 'bar', 'from': 'baz'}))
        # pylint: enable=protected-access



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This is an example CherryPy based event catch used in testing."""
import cherrypy


# pylint: disable=too-few-public-methods
class Root(object):
    """Example root cherrypy class, method dispatcher required."""

    exposed = True

    @cherrypy.tools.json_in()
    @cherrypy.tools.json_out()
    # pylint: disable=invalid-name
    # pylint: disable=no-self-use
    def POST(self):
        """Accept the post data and return it."""
        return cherrypy.request.json


if __name__ == '__main__':
    cherrypy.quickstart(Root(), '/', {
        '/': {
            'request.dispatch': cherrypy.dispatch.MethodDispatcher()
        }
    })



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This is the render object for the search interface."""
from collections import OrderedDict
from six import text_type
import requests
from .config import get_config
from .admin import AdminPolicy

ELASTIC_INDEX = get_config().get('elasticsearch', 'index')
CACHE_SIZE = get_config().getint('policy', 'cache_size')
RELEASER_UUID = AdminPolicy().get_relationship_info(name='authorized_releaser')[0].get('uuid')
SEARCH_REQUIRED_UUID = AdminPolicy().get_relationship_info(name='search_required')[0].get('uuid')


class LimitedSizeDict(OrderedDict):
    """Limited caching dictionary."""

    def __init__(self, *args, **kwds):
        """Constructor for caching dictionary."""
        self.size_limit = kwds.pop('size_limit', None)
        OrderedDict.__init__(self, *args, **kwds)
        self._check_size_limit()

    def __getitem__(self, key):
        """Get the item and put it back so it's on top."""
        val = OrderedDict.__getitem__(self, key)
        try:
            del self[key]
            OrderedDict.__setitem__(self, key, val)
        except KeyError:  # pragma: no cover can't get this covered
            # the key must have gotten purged...
            pass
        return val

    # pylint: disable=signature-differs
    def __setitem__(self, key, value):
        """Set item foo[key] = value."""
        OrderedDict.__setitem__(self, key, value)
        self._check_size_limit()
    # pylint: enable=signature-differs

    def _check_size_limit(self):
        """Function to set the item and remove old ones."""
        if self.size_limit is not None:
            while len(self) > self.size_limit:
                self.popitem(last=False)


def transsip_transsap_render(trans_obj, render_func, obj_cls, trans_key):
    """Wrapper method to handle transaction relationships."""
    ret = {}
    for trans_rel in ['transsip', 'transsap']:
        try:
            obj = SearchRender.get_obj_by_id(trans_rel, trans_obj['_id'])
            ret[obj[trans_key]] = render_func(
                SearchRender.get_obj_by_id(obj_cls, obj[trans_key]))
        except IndexError:
            pass
    return [value for _key, value in ret.items()]


def trans_has_doi(trans_obj):
    """Return true if the transaction has a doi otherwise false."""
    trans_doi = SearchRender.get_trans_doi(trans_obj['_id'])
    if trans_doi != 'false':
        return 'true'
    return 'false'


def trans_access_url(trans_obj):
    """Figure out the access url for the transaction."""
    trans_doi = SearchRender.get_trans_doi(trans_obj['_id'])
    if trans_doi != 'false':
        return get_config().get('policy', 'doi_url_format').format(doi=trans_doi)
    if SearchRender.get_trans_release(trans_obj['_id']) == 'true':
        return get_config().get('policy', 'release_url_format').format(transaction=trans_obj['_id'])
    return get_config().get('policy', 'internal_url_format').format(transaction=trans_obj['_id'])


def trans_science_themes(trans_obj):
    """Render the science theme from a project."""
    return transsip_transsap_render(
        trans_obj, SearchRender.render_science_theme,
        'projects', 'project'
    )


def trans_projects(trans_obj):
    """Render the projects for a transaction."""
    return transsip_transsap_render(
        trans_obj, lambda x: SearchRender.render('projects', x),
        'projects', 'project'
    )


def trans_inst_groups(trans_obj):
    """Render the instrument groups for a transaction."""
    try:
        obj = SearchRender.get_obj_by_id('transsip', trans_obj['_id'])
        return SearchRender.get_groups_from_instrument(obj['instrument'])
    except IndexError:
        pass
    return []


def trans_instruments(trans_obj):
    """Render the instruments for a transaction."""
    try:
        obj = SearchRender.get_obj_by_id('transsip', trans_obj['_id'])
        return [
            SearchRender.render(
                'instruments',
                SearchRender.get_obj_by_id('instruments', obj['instrument'])
            )
        ]
    except IndexError:
        pass
    return []


def trans_institutions(trans_obj):
    """Render the institutions for a transaction."""
    ret = []
    for trans_rel in ['transsip', 'transsap']:
        try:
            obj = SearchRender.get_obj_by_id(trans_rel, trans_obj['_id'])
            ret.extend(SearchRender.get_institutions_from_user(
                obj['submitter']))
        except IndexError:
            pass
    return ret


def trans_users(trans_obj):
    """Render the users list for transactions."""
    return transsip_transsap_render(
        trans_obj, lambda x: SearchRender.render('users', x),
        'users', 'submitter'
    )


def instrument_kvp(inst_obj):
    """Generate a list of key value pairs for transaction."""
    return SearchRender.get_instrument_kvp(inst_obj['_id'])


def trans_kvp(trans_obj):
    """Generate a list of key value pairs for transaction."""
    return SearchRender.get_trans_kvp(trans_obj['_id'])


def user_release(obj):
    """Render the transaction release attribute."""
    return SearchRender.get_user_release(obj['_id'])


def proj_release(obj):
    """Render the transaction release attribute."""
    return SearchRender.get_proj_release(obj['_id'])


def trans_release(obj):
    """Render the transaction release attribute."""
    return SearchRender.get_trans_release(obj['_id'])


# pylint: disable=too-many-public-methods
class SearchRender(object):
    """Search render class to contain methods."""

    obj_cache = LimitedSizeDict(size_limit=CACHE_SIZE)
    render_data = {
        'values': {
            'obj_id': text_type('values_{_id}'),
            'value': text_type('{value}'),
            'display_name': text_type('{display_name}'),
            'description': text_type('{description}'),
            'release': text_type('true')
        },
        'keys': {
            'obj_id': text_type('keys_{_id}'),
            'key': text_type('{key}'),
            'display_name': text_type('{display_name}'),
            'description': text_type('{description}'),
            'release': text_type('true')
        },
        'instruments': {
            'obj_id': text_type('instruments_{_id}'),
            'display_name': text_type('{display_name}'),
            'long_name': text_type('{name}'),
            'key_value_pairs': instrument_kvp,
            'keyword': text_type('{display_name}'),
            'release': text_type('true')
        },
        'institutions': {
            'obj_id': text_type('institutions_{_id}'),
            'display_name': text_type('{name}'),
            'keyword': text_type('{name}'),
            'release': text_type('true')
        },
        'users': {
            'obj_id': text_type('users_{_id}'),
            'display_name': text_type('{last_name}, {first_name} {middle_initial}'),
            'keyword': text_type('{last_name}, {first_name} {middle_initial}'),
            'release': user_release
        },
        'projects': {
            'obj_id': text_type('projects_{_id}'),
            'display_name': text_type('{title}'),
            'long_name': text_type(''),
            'abstract': text_type('{abstract}'),
            'title': text_type('{title}'),
            'keyword': text_type('{title}'),
            'release': proj_release,
            'updated_date': text_type('{updated}'),
            'created_date': text_type('{created}'),
            'closed_date': text_type('{closed_date}'),
            'actual_end_date': text_type('{actual_end_date}'),
            'actual_start_date': text_type('{actual_start_date}')
        },
        'groups': {
            'obj_id': text_type('groups_{_id}'),
            'display_name': text_type('{name}'),
            'keyword': text_type('{name}'),
            'release': text_type('true')
        },
        'transactions': {
            'obj_id': text_type('transactions_{_id}'),
            'access_url': trans_access_url,
            'has_doi': trans_has_doi,
            'users': trans_users,
            'institutions': trans_institutions,
            'instruments': trans_instruments,
            'instrument_groups': trans_inst_groups,
            'projects': trans_projects,
            'science_themes': trans_science_themes,
            'release': trans_release,
            'key_value_pairs': trans_kvp,
            'updated_date': text_type('{updated}'),
            'created_date': text_type('{created}')
        }
    }

    global_get_args = {
        'recursion_depth': '0',
        'recursion_limit': '1'
    }

    @classmethod
    def merge_get_args(cls, get_args):
        """Change a hash of get args and global get args into string for url."""
        get_args.update(cls.global_get_args)
        get_list = []
        for key, val in get_args.items():
            get_list.append(text_type('{}={}').format(key, val))
        return '&'.join(get_list)

    @classmethod
    def get_obj_by_id(cls, obj, obj_id):
        """Get the user from metadata and put it in cache."""
        key = text_type('{}_{}').format(obj, obj_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val
        url = '{base_url}/{obj}?'+cls.merge_get_args({'_id': '{obj_id}'})
        resp = requests.get(
            text_type(url).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                obj=obj,
                obj_id=obj_id
            )
        )
        cls.obj_cache[key] = resp.json()[0]
        return cls.obj_cache[key]

    @classmethod
    def get_trans_doi(cls, trans_id):
        """Get the transaction doi and return false or doi."""
        resp = requests.get(
            text_type('{base_url}/transaction_user?{args}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                args=cls.merge_get_args({'transaction': trans_id, 'relationship': RELEASER_UUID})
            )
        )
        assert resp.status_code == 200
        if not resp.json():
            return 'false'
        rel_uuid = resp.json()[0].get('uuid')
        resp = requests.get(
            text_type('{base_url}/doi_transaction?{args}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                args=cls.merge_get_args({'transaction': rel_uuid})
            )
        )
        resp_json = resp.json()
        if resp_json:
            return resp_json[0].get('doi', 'false')
        return 'false'

    @classmethod
    def get_trans_release(cls, trans_id):
        """Get the transaction release and return true/false."""
        resp = requests.get(
            text_type('{base_url}/transaction_user?{args}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                args=cls.merge_get_args({'transaction': trans_id, 'relationship': RELEASER_UUID})
            )
        )
        if resp.json():
            return 'true'
        return 'false'

    @classmethod
    def get_instrument_kvp(cls, inst_id):
        """Get the instrument key value pairs."""
        resp = requests.get(
            text_type('{base_url}/instrument_key_value?{args}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                args=cls.merge_get_args({'instrument': inst_id, 'relationship': SEARCH_REQUIRED_UUID})
            )
        )
        kvp_hash = {}
        kvp_objs = []
        for kvp_obj in resp.json():
            key_name = cls.get_obj_by_id('keys', kvp_obj.get('key')).get('key')
            value_name = cls.get_obj_by_id('values', kvp_obj.get('value')).get('value')
            kvp_hash[key_name] = value_name
            kvp_objs.append({'key': key_name, 'value': value_name})
        return {
            'key_value_hash': kvp_hash,
            'key_value_objs': kvp_objs
        }

    @classmethod
    def get_trans_kvp(cls, trans_id):
        """Get the transaction key value pairs."""
        resp = requests.get(
            text_type('{base_url}/transactioninfo/by_id/{trans_id}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                trans_id=trans_id
            )
        )
        kvp_hash = resp.json().get('key_values', {})
        return {
            'key_value_hash': kvp_hash,
            'key_value_objs': [{'key': key, 'value': value} for key, value in kvp_hash.items()]
        }

    @classmethod
    def get_proj_release(cls, proj_id):
        """Get the project release from transactions on that proj."""
        for trans_rel in ['transsip', 'transsap']:
            resp = requests.get(
                text_type('{base_url}/{trans_rel}?{args}').format(
                    trans_rel=trans_rel,
                    base_url=get_config().get('metadata', 'endpoint_url'),
                    args=cls.merge_get_args({'project': proj_id})
                )
            )
            for trans_obj in resp.json():
                if cls.get_trans_release(trans_obj['_id']) == 'true':
                    return 'true'
        return 'false'

    @classmethod
    def get_user_release(cls, user_id):
        """Get the user release from transactions on that proj."""
        for trans_rel in ['transsip', 'transsap']:
            resp = requests.get(
                text_type('{base_url}/{trans_rel}?{args}').format(
                    trans_rel=trans_rel,
                    base_url=get_config().get('metadata', 'endpoint_url'),
                    args=cls.merge_get_args({'submitter': user_id})
                )
            )
            for trans_obj in resp.json():
                if cls.get_trans_release(trans_obj['_id']) == 'true':
                    return 'true'
        return 'false'

    @classmethod
    def get_institutions_from_user(cls, user_id):
        """Get an institution list based on user id."""
        key = text_type('inst_by_user_{}').format(user_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        resp = requests.get(
            text_type('{base_url}/institution_user?{args}').format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                args=cls.merge_get_args({'user': user_id})
            )
        )
        ret = []
        for inst_id in [obj['institution'] for obj in resp.json()]:
            ret.append(cls.render('institutions',
                                  cls.get_obj_by_id('institutions', inst_id)))
        cls.obj_cache[key] = ret
        return ret

    @classmethod
    def get_groups_from_instrument(cls, inst_id):
        """Get the list of groups from an instrument."""
        key = text_type('grp_by_inst_{}').format(inst_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        url = '{base_url}/instrument_group?' + \
            cls.merge_get_args({'instrument': '{inst_id}'})
        resp = requests.get(
            text_type(url).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                inst_id=inst_id
            )
        )
        ret = []
        for grp_id in [obj['group'] for obj in resp.json()]:
            ret.append(cls.render(
                'groups', cls.get_obj_by_id('groups', grp_id)))
        cls.obj_cache[key] = ret
        return ret

    @classmethod
    def get_transactions_from_keys(cls, _key_id):
        """Get a list of transactions for a specific key."""
        return []

    @classmethod
    def get_transactions_from_values(cls, _value_id):
        """Get a list of transactions for a specific value."""
        return []

    # pylint: disable=invalid-name
    @classmethod
    def get_transactions_from_institutions(cls, inst_id):
        """Get a list of transactions from an institution."""
        key = text_type('trans_by_instit_{}').format(inst_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        url = '{base_url}/institution_user?' + \
            cls.merge_get_args({'institution': '{inst_id}'})
        resp = requests.get(
            text_type(url).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                inst_id=inst_id
            )
        )
        ret = []
        for user_id in [obj['user'] for obj in resp.json()]:
            ret.extend(cls.get_transactions_from_users(user_id))
        cls.obj_cache[key] = ret
        return ret
    # pylint: enable=invalid-name

    @classmethod
    def get_transactions_from_users(cls, user_id):
        """Get a list of transactions for a user."""
        key = text_type('trans_by_user_{}').format(user_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        ret = set()
        for trans_rel in ['transsip', 'transsap']:
            url = '{base_url}/{trans_rel}?' + \
                cls.merge_get_args({'submitter': '{user_id}'})
            resp = requests.get(
                text_type(url).format(
                    trans_rel=trans_rel,
                    base_url=get_config().get('metadata', 'endpoint_url'),
                    user_id=user_id
                )
            )
            ret.update(set(['transactions_{}'.format(
                obj['_id']) for obj in resp.json()]))
        cls.obj_cache[key] = list(ret)
        return cls.obj_cache[key]

    @classmethod
    def get_transactions_from_projects(cls, proj_id):
        """Get a list of transactions for a project."""
        key = text_type('trans_by_proj_{}').format(proj_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        ret = set()
        for trans_rel in ['transsip', 'transsap']:
            url = '{base_url}/{trans_rel}?' + \
                cls.merge_get_args({'project': '{proj_id}'})
            resp = requests.get(
                text_type(url).format(
                    trans_rel=trans_rel,
                    base_url=get_config().get('metadata', 'endpoint_url'),
                    proj_id=proj_id
                )
            )
            ret.update(set(['transactions_{}'.format(
                obj['_id']) for obj in resp.json()]))
        cls.obj_cache[key] = list(ret)
        return cls.obj_cache[key]

    # pylint: disable=invalid-name
    @classmethod
    def get_transactions_from_instruments(cls, inst_id):
        """Get a list of transactions for a instrument."""
        key = text_type('trans_by_inst_{}').format(inst_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        url = '{base_url}/transsip?' + \
            cls.merge_get_args({'instrument': '{inst_id}'})
        resp = requests.get(
            text_type(url).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                inst_id=inst_id
            )
        )
        cls.obj_cache[key] = ['transactions_{}'.format(
            obj['_id']) for obj in resp.json()]
        return cls.obj_cache[key]
    # pylint: enable=invalid-name

    @classmethod
    def get_transactions_from_groups(cls, group_id):
        """Get a list of instruments for a group."""
        key = text_type('trans_by_group_{}').format(group_id)
        val = cls.obj_cache.get(key, None)
        if val is not None:  # pragma: no cover
            return val

        url = '{base_url}/instrument_group?' + \
            cls.merge_get_args({'group': '{group_id}'})
        resp = requests.get(
            text_type(url).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                group_id=group_id
            )
        )
        ret = []
        for inst_id in [obj['instrument'] for obj in resp.json()]:
            ret.extend(cls.get_transactions_from_instruments(inst_id))
        cls.obj_cache[key] = ret
        return ret

    # pylint: disable=invalid-name
    @classmethod
    def get_transactions_from_science_theme(cls, science_theme):
        """Get a list of transactions for a science theme."""
        key = text_type('trans_by_sci_{}').format(science_theme)
        val = cls.obj_cache.get(key, None)
        if val is not None:
            return val

        url = '{base_url}/projects?' + \
            cls.merge_get_args({'science_theme': '{science_theme}'})
        resp = requests.get(
            text_type(url).format(
                base_url=get_config().get('metadata', 'endpoint_url'),
                science_theme=science_theme
            )
        )
        ret = []
        for proj_id in [obj['_id'] for obj in resp.json()]:
            ret.extend(cls.get_transactions_from_projects(proj_id))
        cls.obj_cache[key] = ret
        return ret
    # pylint: enable=invalid-name

    @classmethod
    def generate(cls, obj_cls, objs, trans_ids=False, render_release=False):
        """generate the institution object."""
        for obj in objs:
            yield {
                '_op_type': 'update',
                '_index': ELASTIC_INDEX,
                '_type': 'doc',
                '_id': text_type('{}_{}').format(obj_cls, obj['_id']),
                'doc': cls.render(obj_cls, obj, trans_ids, render_release),
                'doc_as_upsert': True
            }
            if obj_cls == 'projects':
                yield {
                    '_op_type': 'update',
                    '_index': ELASTIC_INDEX,
                    '_type': 'doc',
                    '_id': text_type('science_theme_{}').format(obj['science_theme']),
                    'doc': cls.render_science_theme(obj, trans_ids),
                    'doc_as_upsert': True
                }

    @classmethod
    def render_science_theme(cls, obj, trans_ids=False):
        """Render the science theme as an object..."""
        ret = {
            'type': 'science_theme',
            'obj_id': text_type('science_theme_{}').format(obj['science_theme']),
            'display_name': obj['science_theme']
        }
        if trans_ids:
            ret['transaction_ids'] = cls.get_transactions_from_science_theme(
                obj['science_theme']
            )
        return ret

    @classmethod
    def render(cls, obj_cls, obj, trans_ids=False, render_release=False):
        """Render the instrument object hash."""
        ret = {
            'type': obj_cls
        }
        for key, value in cls.render_data[obj_cls].items():
            if key == 'release' and not render_release:
                continue
            if callable(value):
                ret[key] = value(obj)
            else:
                if value.format(**obj) == 'None':
                    ret[key] = None
                else:
                    ret[key] = value.format(**obj)
        if trans_ids:
            trans_func = getattr(
                cls, 'get_transactions_from_{}'.format(obj_cls))
            ret['transaction_ids'] = trans_func(obj['_id'])
        return ret
# pylint: enable=too-many-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test the admin command line."""
from unittest import TestCase
import requests
from pacifica.policy.admin_cmd import main
from pacifica.policy.search_render import LimitedSizeDict, SearchRender


class TestAdminCMD(TestCase):
    """Test the admin command line tools."""

    def test_default_search_sync(self):
        """Test the data release subcommand."""
        main('searchsync', '--objects-per-page', '4', '--threads', '1')
        SearchRender.obj_cache = LimitedSizeDict(size_limit=4)
        main('searchsync', '--objects-per-page', '4', '--threads', '1')
        resp = requests.get('http://localhost:9200/pacifica_search/_stats')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(
            resp.json()['indices']['pacifica_search']['primaries']['docs']['count'], 33)

    def test_trans_data_release(self):
        """Test transaction data release."""
        main('--verbose', 'data_release', '--keyword', 'transactions.created',
             '--time-after', '365 days after')
        resp = requests.get('http://localhost:8121/transactions?_id=1234')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['suspense_date'], '2018-07-15')
        resp = requests.get(
            'http://localhost:8121/transaction_user?transaction=1234')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['transaction'], 1234)
        resp = requests.get(
            'http://localhost:8121/transaction_user?transaction=1235')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(len(resp.json()), 0)

    def test_default_data_release(self):
        """Test the data release subcommand."""
        main('data_release', '--time-after',
             '365 days after', '--exclude', u'1234cé')
        resp = requests.get('http://localhost:8121/projects?_id=1234b%C3%A9')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['suspense_date'], '2017-12-10')
        resp = requests.get(
            'http://localhost:8121/transaction_user?transaction=1234')
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json()[0]['transaction'], 1234)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
This is the policy module.

This module is organized by workflow type used by each subproject
of the overall Pacifica software... So there's an uploader module
under which are queries that you can apply policy to to change the
behavour of the uploader.
"""
from __future__ import print_function, absolute_import
from time import sleep
from threading import Thread
from argparse import ArgumentParser, SUPPRESS
import cherrypy
from pacifica.policy.root import Root, error_page_default
from pacifica.policy.globals import CHERRYPY_CONFIG


def stop_later(doit=False):
    """Used for unit testing stop after 10 seconds."""
    if not doit:  # pragma: no cover
        return

    def sleep_then_exit():
        """sleep for 10 seconds then call cherrypy exit."""
        sleep(10)
        cherrypy.engine.exit()
    sleep_thread = Thread(target=sleep_then_exit)
    sleep_thread.daemon = True
    sleep_thread.start()


def main():
    """Main method to start the httpd server."""
    parser = ArgumentParser(description='Run the policy server.')
    parser.add_argument(
        '-c', '--config', metavar='CONFIG', type=str, default=CHERRYPY_CONFIG,
        dest='config', help='cherrypy config file'
    )
    parser.add_argument(
        '-p', '--port', metavar='PORT', type=int, default=8181,
        dest='port', help='port to listen on'
    )
    parser.add_argument(
        '-a', '--address', metavar='ADDRESS', default='localhost',
        dest='address', help='address to listen on'
    )
    parser.add_argument(
        '--stop-after-a-moment', help=SUPPRESS, default=False,
        dest='stop_later', action='store_true'
    )
    args = parser.parse_args()
    stop_later(args.stop_later)
    cherrypy.config.update({'error_page.default': error_page_default})
    cherrypy.config.update({
        'server.socket_host': args.address,
        'server.socket_port': args.port
    })
    cherrypy.quickstart(Root(), '/', args.config)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from pacifica.policy.status.user.search import UserSearch
from pacifica.policy.status.user.lookup import UserLookup


# pylint: disable=too-few-public-methods
class UserQuery(object):
    """CherryPy root object class."""

    exposed = True

    def __init__(self):
        """Create local objects for sub tree items."""
        self.search = UserSearch()
        self.by_id = UserLookup()
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""CherryPy Status Policy object class."""
from cherrypy import tools
import requests
from pacifica.policy.config import get_config
from pacifica.policy.status.base import QueryBase


# pylint: disable=too-few-public-methods
class ProjectKeywordSearch(QueryBase):
    """Retrieves a set of projects for a given keyword set."""

    exposed = True

    def _get_projects_for_keywords(self, user_id, search_terms=None):
        """Return a list with all the projects involving this user."""
        is_admin = self._is_admin(user_id) if user_id is not None else False
        if not search_terms and user_id:
            # no terms, just get any projects for this user
            md_url = '{0}/projectinfo/by_user_id/{1}'.format(
                get_config().get('metadata', 'endpoint_url'), user_id)
        else:
            md_url = '{0}/projectinfo/search/{1}'.format(
                get_config().get('metadata', 'endpoint_url'), search_terms)

        results = []

        response = requests.get(url=md_url)
        if int(response.status_code / 100) == 2:
            if is_admin:
                results = response.json()
            else:
                available_projects = self._get_available_projects(
                    user_id)
                results = [x for x in response.json() if x.get('id')
                           in available_projects]

        return results

    # CherryPy requires these named methods
    # Add HEAD (basically Get without returning body
    # pylint: disable=invalid-name
    @tools.json_out()
    def GET(self, search_terms=None, **kwargs):
        """CherryPy GET method."""
        user_id = kwargs['user'] if 'user' in kwargs else None
        return self._get_projects_for_keywords(user_id, search_terms)
# pylint: enable=too-few-public-methods



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This is used to generically load the test data."""
from os import getenv
from os.path import dirname, realpath, join
from json import loads
from pacifica.metadata.client import PMClient


def main():
    """Main method for loading the test data."""
    mdclient = PMClient(getenv('METADATA_URL', 'http://127.0.0.1:8121'))
    test_data_dir = dirname(realpath(__file__))
    object_order = [
        'users',
        'projects',
        'instruments',
        'instrument_group',
        'instrument_user',
        'project_instrument',
        'project_user',
        'transactions',
        'transsip',
        'transaction_user',
        'doi_entries',
        'doi_transaction',
        'instrument_key_value'
    ]
    for obj in object_order:
        mdclient.create(obj, loads(
            open('{0}.json'.format(join(test_data_dir, obj))).read()))


if __name__ == '__main__':
    main()
